<?
/*
 * compile.inc.php, part of gween
 * Copyright (C) 2002 Alexander Kreuzer <alex@freesources.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/*
Class Syntax
Expression: [Text][(Variable|Function|InlinePHP)][Expression]

Text: output of plain Text
Variable: output of <& phpExpression &>
Function: <%funcName params> </%funcName> 
InlinePHP: % PHP-Code
*/

define(GWEEN_METHOD_NO_OUTPUT, "GWEEN_METHOD_NO_OUTPUT");

function compText($txt, &$call_part, $sl, $el) {
   $txt = preg_replace("/\n/", "\\n", $txt);
   $txt = preg_replace("/\"/", "\\\"", $txt);
   $txt = preg_replace("/\\\$/", '\\\$', $txt);
   $call_part  .= "\$out .= \"$txt\"; //Gweendb:$sl-$el\n"; // quote ', \n and append text ot $out
}

function compVariable($var, &$call_part, $sl, $el) {
   $call_part .= "\$out .= " . compGlobalVar($var) . "; //Gweendb:$sl-$el\n"; // replace vars with the $GOBALS and and append to $out
}

function compFunction($f1, $param, $f2, &$call_part, &$method_part, &$init_part, &$shared_part, $sl, $el) {
   $funcName = preg_replace("/^(\\w+).*$/", "\\1", trim($f1));
   /*if ($funcName != trim($f3)) {
      compilerError("<%$f1> ended with </%$f3>", $sl, $el);
      return;
   }*/

   switch ($funcName) {
      case '': // <%></%> block
	 $call_part .= compGlobalVar($f2) . ";\n";
	 compCheckBE($f2, $sl, $el);
	 break;
      case 'method':
	 $name = preg_replace("/^.*name=\"(.*)\".*$/", "\\1", $param);
	 if ($name == "") {
	    compilerError("method name error in", $sl, $el);
	    return;
	 }
	 compExp($f2, $m_call_part, $m_method_part, $m_init_part, $m_shared_part, $sl); // Compile method as if it
	 if ($m_method_part || $m_init_part || $m_shared_part) {                   // were a normal class
	    compilerError("in methods no method, init or shared parts are allowed<br>", $sl, $el);
	    return;
	 }
	 $method_part .= "ca" . "se '$name':"; // "ca" . "se" for editor reasons :-(
	 $method_part .= $m_call_part;
	 $method_part .= "break;\n";
	 break;
      case 'init':
	 $init_part .= $f2 . "; //Gweendb:$sl-$el\n";
	 compCheckBE($f2, $sl, $el);
	 break;
      case 'shared':
	 $shared_part .= $f2 . "; //Gweendb:$sl-$el\n";
	 compCheckBE($f2, $sl, $el);
	 break;
      case 'doc':
	 break;
      default:
	 compilerError("$funcName is no valid function", $sl, $el);
	 return;
   }
}

function compInlinePHP($f, &$call_part, $sl, $el) { 
   $call_part .= compGlobalVar($f) . " //Gweendb:$sl-$el\n"; 
} 

function compExp($exp, &$call_part, &$method_part, &$init_part, &$shared_part, $lines = 1) {
   $Matches = array();
   if (preg_match("/^(.*?)(?:<&(.*?)&>|<%(.*?)(?:[ ]+(.*?))?>(.*?)<\/%\\3>|(?:^\n*|\n)%(.*?)(?=\n))(.*)$/s", $exp, $Matches)) { 

      $tl = $lines;
      for ($i = 1; $i < 6; $i++) {
	 $tc = preg_match_all("/\n/", $Matches[$i], $T);
	 //echo $i, "<br>";
	 if ($i != 5) compCheckBE($Matches[$i], $tl, $tl + $tc);
	 $tl += $tc;
      }
      unset($tl);
      unset($tc);

      if ($Matches[1]) {
	 compText($Matches[1], $call_part, $lines, $lines += preg_match_all("/\n/", $Matches[1], $T));
      }
      if ($Matches[2]) {
	 compVariable($Matches[2], $call_part, $lines, $lines += preg_match_all("/\n/", $Matches[2], $T));
      }
      if ($Matches[3] || $Matches[4] || $Matches[5]) {
	 compFunction($Matches[3],
		      $Matches[4],
		      $Matches[5],
		      $call_part,
		      $method_part,
		      $init_part,
		      $shared_part,
		      $lines,
		      $lines += preg_match_all("/\n/", $Matches[4], $T));
      }
      if ($Matches[6]) {
	 compInlinePHP($Matches[6], $call_part, $lines += 1, $lines);
      }
      if ($Matches[7]) {
	 compExp($Matches[7], $call_part, $method_part, $init_part, $shared_part, $lines);
      }
   }
   else {
      compText($exp, $call_part, $lines, $lines += preg_match_all("/\n/", $exp, $T));
   }
}

function compCheckBE($s, $sl, $el) { // checks for lonley <%|<\/%|\n%|<&|&> and triggers error if needed
   if (preg_match("/(<%|<\/%|\n%|<&|&>)/", $s)) {
      echo $s, "<br>";
      compilerError("Tag open and not closed\n", $sl, $el);
   }
}

function compGlobalVar($s) {
   $r = "";
   $instr = false;
   $instr2 = false;
   for($i = 0; $i < strlen($s); $i++) {
      $c = substr($s, $i, 1);
      switch ($c) {
	 case '\\':
	    $r .= $c;
	    $c = substr($s, ++$i, 1);
	    $r .= $c;
	    break;
	 case '"':
	    if (!$instr) $instr2 = !$instr2;
	    $r .= $c;
	    break;
	 case '\'':
	    if (!$instr2) $instr = !$instr;
	    $r .= $c;
	    break;
	 case '$':
	    if (!$instr) {
               $cache = "";
	       while (preg_match("/\\w/", ($c = substr($s, ++$i, 1)))) {
		  $cache .= $c;
	       }
	       
	       if(! empty($cache)) {
                  //if($instr2) $r .= '".$GLOBALS[\''.$cache.'\']."' . $c;
                  //else $r .= '$GLOBALS[\''.$cache.'\']' . $c;
		  $r .= '$GLOBALS[\''.$cache.'\']' . $c;
               }
               else $r .= '$' . $c;
	    }
	    else {
	       $r .= $c;
	    }
	    break;
	 default:
	    $r .= $c;
      }
   }
   return $r;
}

function compilerError($msg, $sl, $el) {
   global $Comp_in, $comp_in_file;
   logError(E_USER_ERROR, "$comp_in_file:" . ($sl+1) . '-' . ($el+1) . ":\n$msg", array_splice($Comp_in, $sl, $el - $sl), array()); 
   $GLOBALS["comp_error"] = true;
}

function compile($in_file, $out_file) {
   global $Comp_in, $comp_error;
   $comp_error = false;

   $GLOBALS["comp_in_file"] = $in_file;
   $Comp_in = file($in_file);
   $in_c = implode("", $Comp_in);

   $out = fopen($out_file, "w");

   $compName = "comp" . preg_replace("/(\/|\\.|-)/", "_", $in_file) . time();

   $method_part = "switch(\$method) {\n";

   compExp($in_c, $call_part, $method_part, $init_part, $shared_part);

   if ($comp_error) {
      fclose($out);
      unlink($out_file);
      return 1;
   }

   $method_part .= "default:\n";
   $method_part .= "\$out = GWEEN_METHOD_NO_OUTPUT;";
   $method_part .= "}\n";
   $method_part .= "return \$out;\n";

   $out_c = "<?\n";
   $out_c .= "require_once(GWEEN_DIR.'comp.class.php');\n";

   $out_c .= $shared_part;

   $out_c .= "class $compName extends Component {\n";

   $out_c .= "function $compName() {\n";
   $out_c .= $init_part;
   $out_c .= "}\n";

   $out_c .= "function callMethod(\$method) {\n";
   $out_c .= "\$out = \"\";\n";
   $out_c .= $method_part;
   $out_c .= "}\n";
  
   $out_c .= "function call() {\n";
   $out_c .= "\$out = \"\";\n";
   $out_c .= $call_part;
   $out_c .= "return \$out;\n";
   $out_c .= "}\n";

   $out_c .= "}\n";
   $out_c .= "return(new $compName())\n";
   $out_c .= "?>";

   $out_c = preg_replace("/;(\s*)\{/", "\\1{", $out_c);
   //$out_c = preg_replace("/(\{|\})(\s*);/", "\\1\\2", $out_c);

   fwrite($out, $out_c);
   fclose($out);
   return 0;
}
?>
