<?
/*
 * debug.inc.php, part of gween
 * Copyright (C) 2002 Alexander Kreuzer <alex@freesources.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

// These functions are intended as utility-functions for parser.class.php
// and to provide an errorHandler.
// They should only be call from parser.class.php or by triggering an error
// execpt logError this function can be call from outside if more control over the error
// output is needed.

set_error_handler("errorHandler");
$deb_errorCount = 0;

function n2br ($s) {
   return preg_replace("/\n/", "<br>", $s);
}

function errName($errno) {
   switch($errno) {
      case E_ERROR:
      case E_USER_ERROR:
	 return "Error";
	 break;
      case E_WARNING:
      case E_USER_WARNING:
	 return "Warning";
	 break;
      case E_NOTICE:
      case E_USER_NOTICE:
	 return "Notice";
	 break;
   }
}

function errorHandler($errno, $errstr, $errfile, $errline) {
   if (preg_match("!" . CACHE_PATH . "(.*)\.php$!", $errfile, $Match)) {
      $compErrFile = COMPONENT_PATH . $Match[1];
      $F = file($errfile);
      $i = $errline;
      while(!preg_match("/\/\/Gweendb:(\d+)-(\d+)/", $F[$i], $Match) and $i < count($F)) {
	 $i++;
      }
      $compErrLineS = $Match[1];
      $compErrLineE = $Match[2];
      $f_start = ($compErrLineS-5 < 0 ? 0 : $compErrLineS-5);
      
      logError($errno,
	       "$compErrFile:$compErrLineS-$compErrLineE",
	       array_splice(file($compErrFile), $f_start, $compErrLineE - $compErrLineS + 11),
	       range($compErrLineS - 1 - $f_start, $compErrLineE - 1 - $f_start),
	       $errno == E_ERROR || $errno == E_USER_ERROR);
   }
   else {
      $f_start = ($errline-5 < 0 ? 0 : $errline-5);
      logError($errno,
	       "$errfile:$errline\n$errstr",
	       array_splice(file($errfile), $f_start, 10),
	       array($errline - 1 - $f_start),
	       $errno == E_ERROR || $errno == E_USER_ERROR);
   }
}



function logError($errno, $errstr, $Lines, $Highlight = array(), $stop = false) {
   global $deb_errorOut;
   $deb_errorOut .= '<p class="error_des">' . errName($errno) . ': ' . n2br(htmlentities($errstr)) . '</p>';
   $deb_errorOut .= '<p class="code">';
   for($i = 0; $i < count($Lines); $i++) {
      if (in_array($i, $Highlight)) {
	 $deb_errorOut .= '<i class="code_highlight">' . htmlentities($Lines[$i]) . '</i><br>';
      }
      else {
	 $deb_errorOut .= htmlentities($Lines[$i]) . '<br>';
      }
   }
   $deb_errorOut .= '</p>';
   if ($errno == E_ERROR || $errno == E_USER_ERROR) {
      $GLOBALS["deb_errorCount"]++;
   }
   if ($stop) {
      echo getErrorConsole();
      exit();
   }
}

function hasError() {
   return $GLOBALS["deb_errorCount"] != 0;
}

function getErrorConsole() {
   global $deb_errorOut, $deb_errorCount;
   return
"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">
<html>
<head>
<title>DebugConsole</title>
<style type=\"text/css\">
p.error_des { font-size:large }
p.code { font-size:small }
i.code_highlight { color:red }
</style>
</head>
<body>
<h1>DebugConsole</h1>
<h4>$deb_errorCount Error(s)</h4>
$deb_errorOut
</body></html>";
}

?>
