<?
/*
 * parser.inc.php, part of gween
 * Copyright (C) 2002 Alexander Kreuzer <alex@freesources.org>,
 *                    Moritz Sinn <moritz@freesources.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

// Class Parser
// This class call compiler and then parses the components together

require_once(GWEEN_DIR."compiler.inc.php");
require_once(GWEEN_DIR."debug.inc.php");

class Parser {
   var $Hierarchy = array();     // Component Hierarchy (comp names)
   var $CompHierarchy = array(); // Component Hierarchy (comp objects)
   var $index = -1;
   var $path_index = array();
   var $self;   // self: call componet
   var $parent; // parent: parent of self
   var $thisComp; // thisComp: points to the currently process component
   var $thisStack = array();
   var $Components = array(); // hold all already loaded components
   var $comp_path;
   var $cache_path;
   var $debug;

   function Parser($comp_n, $comp_path, $cache_path, $debug) {
      $this->comp_path = $comp_path;
      $this->cache_path = $cache_path;
      $this->debug = $debug;
      if(empty($comp_n)) {
	 $comp_n = "index.comp";
      }
      else if(is_dir($this->comp_path . $comp_n)) {
	 $comp_n = preg_replace("/^(.*?)\/?$/", "\\1/index.comp", $comp_n);
      }
      if(!file_exists($this->comp_path . $comp_n))
      {
	 trigger_error("Component $comp_n does not exist", E_USER_ERROR);
      }

      $this->Hierarchy[] = $comp_n; // fill Hierarchy with component an autohandlers
      $this->path_index[] = 0;
      $path_depth = 0;
      while ($comp_n != "") {
	 //$this->path_index[$i]++;
	 $comp_n = dirname($comp_n);
	 if ($comp_n == ".") {
	    $comp_n = "";
	 }
	 $t = ($comp_n == "" ? "" : $comp_n . "/") . "autohandler.comp";
	 if (file_exists($this->comp_path . $t)) {
	    array_unshift($this->path_index, $path_depth);
	    array_unshift($this->Hierarchy, $t);
	 }
	 $path_depth++;
      }

      foreach ($this->Hierarchy as $t) { // fill CompHierarchy
	 $this->CompHierarchy[] = $this->fetchComp($t);
      }

      $this->self = $this->CompHierarchy[(count($this->CompHierarchy) >= 1 ? count($this->CompHierarchy) - 1 : 0)];
      $this->parent = $this->CompHierarchy[(count($this->CompHierarchy) >= 2 ? count($this->CompHierarchy) - 2 : 0)];
      $this->thisComp = $this->CompHierarchy[0];
   }

   function callNext() {
      if (is_object($this->CompHierarchy[$this->index+1])) {
	 array_push($this->thisStack, $this->thisComp);
	 $this->index++;
	 $this->thisComp = $this->CompHierarchy[$this->index];
	 $r = $this->CompHierarchy[$this->index]->call();
	 $this->index--;
	 $this->thisComp = array_pop($this->thisStack);
	 return $r;
      } 
      else {
	 trigger_error("Could not call component with index " . $this->index+1, E_USER_ERROR);
	 return "";
      }
   }

   function call($comp_n) {
      $t = $this->fetchComp(dirname($this->Hierarchy[$this->index]) . "/" . $comp_n);
      if (is_object($t)) {
	 array_push($this->thisStack, $this->thisComp);
	 $this->thisComp = $t;
	 $r = $t->call();
	 $this->thisComp = array_pop($this->thisStack);
	 return $r;
      }
      else {
	 trigger_error("Could not load component $comp_n", E_USER_ERROR);
	 return "";
      }
   }

   function callMethod($method) {
      for($i = count($this->CompHierarchy) - 1; $i > 0; $i--) {
	 $out = $this->CompHierarchy[$i]->callMethod($method);
	 if ($out != GWEEN_METHOD_NO_OUTPUT) {
	    break;
	 }
      }
      return ($out == GWEEN_METHOD_NO_OUTPUT ? "" : $out);
   }

   function rl($link) { // relativ link use for autohandler to automaticly add ../
      $c = $this->path_index[$this->index];
      for($i = 0; $i < $c; $i++) $link = '../' . $link;
      return $link;
   }

   function start() {
      $out = $this->callNext();
      if ($this->debug) {
	 echo getErrorConsole();
      }
      else {
	 echo $out;
      }
   }

   function getSelfCompName() {
      return $this->Hierarchy[count($this->Hierarchy)-1];
   }

   function fetchComp($comp_n) {
      if (isset($this->Components[$comp_n])) {
	 return($this->Components[$comp_n]);
      }
      else {
	 $in_file = $this->comp_path . $comp_n;
	 $out_file = $this->cache_path . $comp_n . ".php";
	 
	 if (!file_exists($out_file) ||                    // check if compiled component exists
	     filemtime($out_file) < filemtime($in_file) || // and is newer than the source-file
	     GWEEN_ALWAY_RECOMPILE) 
	 {                                                 // compile the component as needed
	    // create new missing directorys in cache directory
	    $OutPath = explode("/", $out_file);
	    array_pop($OutPath);
	    $out_path = "";
	    foreach($OutPath as $dir) {
	       $out_path .= $dir."/";
	       if(! is_dir($out_path))
	       {
		  mkdir($out_path, 0755);
	       }
	    }
	    if (compile($in_file, $out_file) != 0) { // Compiler Error
	       trigger_error("Faild compiling $comp_n", E_USER_ERROR);
	    }	    
	 }
	 $r = include($out_file);
	 if (!is_object($r)) {
	    trigger_error("Faild loading $comp_n", E_USER_ERROR);
	 }
	 $this->Components[$comp_n] = $r;
	 return($r);
      }
   }
}
?>
